<?php

/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require __DIR__ . '/vendor/autoload.php';

class Clippay extends PaymentModule
{
    protected $config_form = false;

    // const URL_FRONTED = 'https://clip-woocommerce-stage.conexa.ai';

    public function __construct()
    {
        $this->name = 'clippay';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.4';
        $this->author = 'conexa';
        $this->need_instance = 0;

        /*
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Clip');
        $this->description = $this->l('Boost your sales with the Checkout Clip module from your online store in Prestashop; it\'s easy, fast and secure and accepts all major credit and debit cards as well as cash at participating convenience stores.');
        $this->limited_countries = ['MX'];
        $this->limited_currencies = ['MXN'];
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');

            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');

            return false;
        }

        Configuration::updateValue('CLIPPAY_LIVE_MODE', false);
        Configuration::updateValue('CLIPPAY_STATE', 'Pay with card or cash');

        $this->getState('CHECKOUT_CREATED', '#2CB7F7', $this->l('CHECKOUT CREATED'));
        $this->getState('CHECKOUT_PENDING', '#2381E3', $this->l('CHECKOUT PENDING'));
        $this->getState('CHECKOUT_CANCELLED', '#FF2303', $this->l('CHECKOUT CANCELLED'));
        $this->getState('CHECKOUT_EXPIRED',   '#FF2303', $this->l('CHECKOUT EXPIRED'));
        $this->getState('CHECKOUT_COMPLETED', '#006822', $this->l('CHECKOUT COMPLETED'), true, true);

        include dirname(__FILE__) . '/sql/install.php';

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('paymentOptions');
    }

    public function uninstall()
    {
        (new OrderState((int) Configuration::get('CLIP_CHECKOUT_CREATED')))->softDelete();
        (new OrderState((int) Configuration::get('CLIP_CHECKOUT_PENDING')))->softDelete();
        (new OrderState((int) Configuration::get('CLIP_CHECKOUT_CANCELLED')))->softDelete();
        (new OrderState((int) Configuration::get('CLIP_CHECKOUT_EXPIRED')))->softDelete();
        (new OrderState((int) Configuration::get('CLIP_CHECKOUT_COMPLETED')))->softDelete();

        Configuration::deleteByName('CLIP_CHECKOUT_CREATED');
        Configuration::deleteByName('CLIP_CHECKOUT_PENDING');
        Configuration::deleteByName('CLIP_CHECKOUT_CANCELLED');
        Configuration::deleteByName('CLIP_CHECKOUT_EXPIRED');
        Configuration::deleteByName('CLIP_CHECKOUT_COMPLETED');
        Configuration::deleteByName('CLIPPAY_LIVE_MODE');
        Configuration::deleteByName('CLIPPAY_STATE');
        Configuration::deleteByName('CLIPPAY_LOG');
        Configuration::deleteByName('CLIPPAY_TOKEN_SUCCESS');
        Configuration::deleteByName('CLIPPAY_SECRET_KEY');
        Configuration::deleteByName('CLIPPAY_APIKEY');
        Configuration::deleteByName('CLIPPAY_XAPIKEY');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /*
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('submitClippayModule')) == true && !empty($_POST)) {
            $this->postProcess();
        }

        /*
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('resetClippayModule')) == true) {
            Configuration::deleteByName('CLIPPAY_SECRET_KEY');
            Configuration::deleteByName('CLIPPAY_APIKEY');
            Configuration::deleteByName('CLIPPAY_XAPIKEY');
        }

        $token = base64_encode(random_bytes(32));
        Configuration::deleteByName('CLIPPAY_TOKEN_SUCCESS');
        Configuration::updateValue('CLIPPAY_TOKEN_SUCCESS', $token);

        $urlReturn = Context::getContext()->link->getModuleLink(
            'clippay',
            'credential',
            ['token' => $token]
        );

        $xapikey = Configuration::get('CLIPPAY_XAPIKEY');

        $urlFormReset = $this->context->link->getAdminLink(
            'AdminModules',
            true,
            [],
            [
                'configure' => 'clippay',
                'tab_module' => 'payments_gateways',
                'module_name' => 'clippay',
            ]
        );

        $json = json_decode(
            file_get_contents($this->local_path . '/views/js/assets/manifest.json'),
            true
        );

        if (!empty($json)) {
            $urlJS = '/modules/clippay/views/js/assets/' . $json['resources/js/main.tsx']['file'];
            $this->context->smarty->assign('url_js', $urlJS);
        }

        if (!empty($json)) {
            $urlCss = '/modules/clippay/views/js/assets/' . $json['resources/js/main.css']['file'];
            $this->context->smarty->assign('url_css', $urlCss);
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('url_return', $urlReturn);
        $this->context->smarty->assign('xapikey', $xapikey);
        $this->context->smarty->assign('url_formReset', $urlFormReset);
        $this->context->smarty->assign('lang', $this->context->language->iso_code);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        if (empty($xapikey)) {
            return $output;
        }

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitClippayModule';
        $helper->currentIndex = $this->context->link->getAdminLink(
            'AdminModules',
            false
        ) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm(
            [
                $this->getStatusesForm(),
            ]
        );
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int) $currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        $order = $params['order'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
            $this->smarty->assign('status', 'ok');
        }

        $this->smarty->assign(
            [
                'shop_name' => [Configuration::get('PS_SHOP_NAME')],
                'id_order' => $order->id,
                'reference' => $order->reference,
                'params' => $params,
                'total' => Tools::displayPrice(
                    $order->total_paid,
                    false
                ),
            ]
        );

        return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $xapikey = Configuration::get('CLIPPAY_XAPIKEY');

        if (empty($xapikey)) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $logo = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/logo.png';

        $stateId = Configuration::get('CLIPPAY_STATE');
        $text = $stateId;
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $isoLanguage = $this->context->language->iso_code;

        $imagen = null;

        if (in_array($isoLanguage,['es', 'mx']) && $stateId == 'Pay with card or cash') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cards_cash_spanish.svg';
            $text = $this->l('Pay with card or cash');
        }

        if (in_array($isoLanguage,['es', 'mx']) && $stateId == 'Pay with card') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cards_spanish.svg';
            $text = $this->l('Pay with card');
        }

        if (in_array($isoLanguage,['es', 'mx']) && $stateId == 'Pay with cash') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cash_spanish.svg';
            $text = $this->l('Pay with cash');
        }

        if ($isoLanguage == 'en' && $stateId == 'Pay with card or cash') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cards_cash_english.svg';
            $text = $this->l('Pay with card or cash');
        }

        if ($isoLanguage == 'en' && $stateId == 'Pay with card') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cards_english.svg';
            $text = $this->l('Pay with card');
        }

        if ($isoLanguage == 'en' && $stateId == 'Pay with cash') {
            $imagen = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/clippay/views/img/cash_english.svg';
            $text = $this->l('Pay with cash');
        }

        $output = $this->context->smarty->fetch(
            $this->local_path . 'views/templates/hook/payment_options.tpl',
            compact('stateId', 'imagen')
        );

        $option->setLogo(
            $logo
        )->setCallToActionText(
            $this->l($stateId)
        )->setAction(
            $this->context->link->getModuleLink(
                $this->name,
                'redirect',
                [],
                true
            )
        )->setAdditionalInformation(
            $output
        );

        return [
            $option,
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    public function getState($key, $color, $lang, $paid = false, $logable = false)
    {
        $status = OrderState::getOrderStates($this->context->language->id);

        $statusNew = null;

        foreach ($status as $item) {
            if ($item['module_name'] == 'clippay' && in_array($item['name'], ['CLIP_' . $key, $lang])) {
                Configuration::updateValue('CLIP_' . $key, $item['id_order_state']);
                return $item['id_order_state'];
            }
        }

        if (is_null($statusNew)) {
            $statusNew = new OrderState();
            $statusNew->color = $color;
            $statusNew->paid = $paid;
            $statusNew->logable = $logable;
            $statusNew->name = ['CLIP_' . $key];
            $statusNew->send_email = false;
            $statusNew->module_name = 'clippay';
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                $statusNew->name[$language['id_lang']] = $lang;
            }
            $statusNew->add();
        }

        Configuration::updateValue('CLIP_' . $key, $statusNew->id);

        return $statusNew->id;
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $data = [];
        $data['CLIPPAY_STATE'] = \Configuration::get('CLIPPAY_STATE', null);
        $data['CLIPPAY_LOG'] = \Configuration::get('CLIPPAY_LOG', null);

        return $data;
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            \Configuration::updateValue($key, \Tools::getValue($key));
        }
    }

    protected function getStatusesForm()
    {
        $id1 = 'Pay with card or cash';
        $id2 = 'Pay with card';
        $id3 = 'Pay with cash';

        $optionsQuery = [
            [
                'id' => $id1,
                'name' => $this->l($id1),
            ],
            [
                'id' => $id2,
                'name' => $this->l($id2),
            ],
            [
                'id' => $id3,
                'name' => $this->l($id3),
            ],
        ];

        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Payment type'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 8,
                        'type' => 'select',
                        'label' => $this->l('Payment type'),
                        'desc' => $this->l('Select the type of payment'),
                        'name' => 'CLIPPAY_STATE',
                        'options' => [
                            'query' => $optionsQuery,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'col' => 8,
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Log'),
                        'desc' => $this->l('Generate logs'),
                        'name' => 'CLIPPAY_LOG',
                        'values' => [
                            [
                                'id' => 'log_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'log_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'name' => 'submitClippayModule',
                    'type' => 'submit',
                ],
            ],
        ];
    }
}
