<?php

/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

use Conexa\Clippay\Helpers\ClippayLog;
use Conexa\Clippay\Repository\ClippayPaymentModel;
use Conexa\Clippay\Services\ClipService;
use Exception;

/**
 *
 * 
 **/
class ClippayWebhookModuleFrontController extends ModuleFrontController
{
    /**
     * @var ClipService
     */
    protected $service;

    /**
     * @var array
     */
    protected $body;

    /**
     * @var string
     */
    protected $clippay;

    /**
     * @var string
     */
    protected $cart_id;

    public function __construct()
    {
        parent::__construct();

        $this->service = new ClipService();

        $this->body = json_decode(Tools::file_get_contents('php://input'), true);

        $this->clippay = Tools::getValue('clippay');

        $this->cart_id = Tools::getValue('cart_id');
    }

    // CHECKOUT_CREATED El link de pago fue creado exitosamente.
    // CHECKOUT_PENDING El link de pago está activo y pendiente de ser completado.
    // CHECKOUT_CANCELLED El link de pago fue cancelado por exceso de intentos (máximo 5).
    // CHECKOUT_EXPIRED El link de pago expiró. El link de pago expira después de 1 hora.
    // CHECKOUT_COMPLETED El link de pago se liquidó.

    public function postProcess()
    {

        try {

            $clippayPaymentId = ClippayPaymentModel::getPaymentClippay($this->clippay);

            if ($clippayPaymentId === false) {
                exit('fail');
            }


            $clippayPayment = new ClippayPaymentModel($clippayPaymentId);

            $checkout = $this->service->statusCheckout($clippayPayment->payment_request_id);

            $status = $checkout['status'];

            ClippayLog::info('WEBHOOK ' . $clippayPayment->id_cart . ' ' . json_encode($checkout));
            ClippayLog::info('-------------->>>> ' . $status);

            switch ($status) {
                case 'CHECKOUT_CREATED':
                    // ... nothing to do
                    break;
                case 'CHECKOUT_PENDING':
                    $this->pending($clippayPayment, $checkout);
                    break;
                case 'CHECKOUT_CANCELLED':
                    $this->cancelled($clippayPayment, $checkout);
                    break;
                case 'CHECKOUT_EXPIRED':
                    $this->expired($clippayPayment, $checkout);
                    break;
                case 'CHECKOUT_COMPLETED':
                    $this->completed($clippayPayment, $checkout);
                    break;
                default:
                    break;
            }

            exit('OK');
        } catch (Exception $e) {
            ClippayLog::error($e);
            throw $e;
        }
    }

    public function pending(&$clippayPayment, &$checkout)
    {
        /*
        * if order exists, cancel
         */
        if ($clippayPayment->id_order) {
            return;
        }

        $order = new Order($clippayPayment->id_order);

        if (!$order->id) {
            ClippayLog::info('CREATE ORDER' . json_encode($clippayPayment));
            $order = $this->createOrder($clippayPayment, $checkout, (int) Configuration::get('CLIP_CHECKOUT_PENDING'));
        }
    }

    public function completed(&$clippayPayment, &$checkout)
    {
        $order = new Order($clippayPayment->id_order);

        if ($order->id) {
            $order->setCurrentState((int) Configuration::get('CLIP_CHECKOUT_COMPLETED'));
            $payments = $order->getOrderPayments();
            $payment = null;

            foreach ($payments as $item) {
                if ($item->payment_method == $this->module->displayName) {
                    $payment = $item;
                    break;
                }
            }

            if (is_null($payment)) {
                $order->addOrderPayment(
                    $checkout['amount'],
                    null,
                    array_key_exists('receipt_no', $checkout) ? $checkout['receipt_no'] : ''
                );
                $clippayPayment->transaction_id = array_key_exists(
                    'receipt_no',
                    $checkout
                ) ? $checkout['receipt_no'] : '';
                $clippayPayment->save();
            }
        }

        if (!$order->id) {
            ClippayLog::info('CREATE ORDER ' . json_encode($clippayPayment));
            $order = $this->createOrder($clippayPayment, $checkout, (int) Configuration::get('CLIP_CHECKOUT_COMPLETED'));
        }
    }

    public function createOrder(&$clippayPayment, &$checkout, $statusId)
    {
        /*
         * if order exists, cancel
         */
        if ($clippayPayment->id_order) {
            ClippayLog::info('......');
            return;
        }

        $cart = new Cart((int) $clippayPayment->id_cart);
        $customer = new Customer((int) $cart->id_customer);
        $currency = new Currency((int) $cart->id_currency);
        $language = new Language((int) $customer->id_lang);
        $secure_key = $customer->secure_key;

        ClippayLog::info(
            json_encode(
                [
                    $cart->id,
                    $statusId,
                    $checkout['amount'],
                    $this->module->displayName,
                    $message = null,
                    [
                        'transaction_id' => array_key_exists(
                            'receipt_no',
                            $checkout
                        ) ? $checkout['receipt_no'] : null
                    ],
                    (int) $currency->id,
                    false,
                    $secure_key
                ]
            )
        );


        $this->module->validateOrder(
            $cart->id,
            $statusId,
            $checkout['amount'],
            $this->module->displayName,
            $message = null,
            [
                'transaction_id' => array_key_exists(
                    'receipt_no',
                    $checkout
                ) ? $checkout['receipt_no'] : null
            ],
            (int) $currency->id,
            false,
            $secure_key
        );

        $order_id = Order::getOrderByCartId((int) $clippayPayment->id_cart);
        $clippayPayment->id_order = $order_id;
        $clippayPayment->transaction_id = array_key_exists(
            'receipt_no',
            $checkout
        ) ? $checkout['receipt_no'] : '';
        $clippayPayment->save();
    }

    public function expired(&$clippayPayment, &$checkout)
    {
        /*
        * if order exists, cancel
         */
        if (!$clippayPayment->id_order) {
            return;
        }

        $order = new Order($clippayPayment->id_order);
        if ($order->id) {
            $order->setCurrentState((int) Configuration::get('CLIP_CHECKOUT_EXPIRED'));
        }
    }

    public function cancelled(&$clippayPayment, &$checkout)
    {
        /*
        * if order exists, cancel
         */
        if (!$clippayPayment->id_order) {
            return;
        }

        $order = new Order($clippayPayment->id_order);
        if ($order->id) {
            $order->setCurrentState((int) Configuration::get('CLIP_CHECKOUT_CANCELLED'));
        }
    }
}
